@@ -8,7 +8,12 @@ from .world_objects import *
 
 # import primitives but don't pull into namespace
 from . import primitives
-from .primitives import Ray, Vector, Point, bundle_of_rays  # bring in basic ray properties
+from .primitives import (
+    Ray,
+    Vector,
+    Point,
+    bundle_of_rays,
+)  # bring in basic ray properties
 from . import renderers
 from . import csg
-from . import materials
\ No newline at end of file
+from . import materials
