@@ -1,6 +1,7 @@
 import collections
 import copy
 
+from itertools import count
 from numpy import linalg as linalg
 from scipy.spatial import transform as transform
 
@@ -23,24 +24,11 @@ def bounding_box(point_set):
 
 
 class CountedObject(object):
-    _object_count = 0  # keeps track of how many objects of each type exist
-    _next_object_id = 0  # the next id number to use when a new object is initialized
-
-    @classmethod
-    def _increment(cls):
-        # increments the classes object count and returns the object number of the instance that called it
-        this_id = cls._next_object_id
-        cls._object_count += 1
-        cls._next_object_id += 1
-        return this_id
-
-    @classmethod
-    def _decrement(cls):
-        cls._object_count -= 1
+    _ids = count(0)
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)  # call the next constructor in the MRO
-        self._id = self._increment()  # object specific id number for this instance
+        self._id = next(self._ids)
 
     def get_id(self):
         """
@@ -51,18 +39,6 @@ class CountedObject(object):
         """
         return self._id
 
-    @classmethod
-    def get_count(cls):
-        """
-        Gets the current count of objects
-        :return: the current number of objects in memory
-        :rtype: int
-        """
-        return cls._object_count
-
-    def __del__(self):
-        type(self)._decrement()  # reduce the object count by 1
-
 
 class WorldObject(CountedObject):
     """
@@ -89,8 +65,8 @@ class WorldObject(CountedObject):
         :return:
         """
         if format == "deg":
-            cos_a = np.cos(angle * np.pi / 180.)
-            sin_a = np.sin(angle * np.pi / 180.)
+            cos_a = np.cos(angle * np.pi / 180.0)
+            sin_a = np.sin(angle * np.pi / 180.0)
 
         elif format == "rad":
             cos_a = np.cos(angle)
@@ -106,47 +82,60 @@ class WorldObject(CountedObject):
         self._obj_origin = primitives.Point(0, 0, 0)  # position in object space
         self._obj_direction = primitives.Vector(0, 0, 1)  # direction in object space
 
-        self._world_origin = primitives.Point(0, 0, 0)  # the objects position in world space
-        self._world_direction = primitives.Vector(0, 0, 1)  # the objects direction in world space
+        self._world_origin = primitives.Point(
+            0, 0, 0
+        )  # the objects position in world space
+        self._world_direction = primitives.Vector(
+            0, 0, 1
+        )  # the objects direction in world space
 
         # Flags that get set to false whenever the transform matrix has been updated
         self._dir_valid = True
         self._pos_valid = True
         self._obj_transform_valid = True
 
-        self._world_coordinate_transform = np.identity(4,
-                                                       dtype=float)  # transform matrix from object to world space
+        self._world_coordinate_transform = np.identity(
+            4, dtype=float
+        )  # transform matrix from object to world space
         self._object_coordinate_transform = np.identity(4, dtype=float)
 
         # var watch list is a list of function to call whenever the world transform matrix is updated
         # the function must accept no arguments
-        self.var_watchlist = [
-            self._world_matrix_update_handler
-        ]
+        self.var_watchlist = [self._world_matrix_update_handler]
 
     def _world_matrix_update_handler(self):
         # update the world origin and direction
 
-        self._world_origin = np.matmul(self._world_coordinate_transform, self._obj_origin)
+        self._world_origin = np.matmul(
+            self._world_coordinate_transform, self._obj_origin
+        )
         world_dir = np.matmul(self._world_coordinate_transform, self._obj_direction)
         norm = linalg.norm(world_dir)
-        if norm < 1E-7:
-            raise ValueError(f"Measured Norm of World primitives.Vector below tolerance: {norm}")
+        if norm < 1e-7:
+            raise ValueError(
+                f"Measured Norm of World primitives.Vector below tolerance: {norm}"
+            )
         else:
             self._world_direction = world_dir / norm
 
         # update the object transform matrix
-        self._object_coordinate_transform = np.linalg.inv(self._world_coordinate_transform)
+        self._object_coordinate_transform = np.linalg.inv(
+            self._world_coordinate_transform
+        )
 
     def _append_world_transform(self, new_transform):
-        self._world_coordinate_transform = np.matmul(new_transform, self._world_coordinate_transform)
+        self._world_coordinate_transform = np.matmul(
+            new_transform, self._world_coordinate_transform
+        )
         # update the functions in the watch list
         [fn() for fn in self.var_watchlist]
 
     def get_position(self):
         # check if the position is valid, if not update it and return
         if not self._pos_valid:
-            self._world_origin = np.matmul(self._world_coordinate_transform, self._obj_origin)
+            self._world_origin = np.matmul(
+                self._world_coordinate_transform, self._obj_origin
+            )
             self._pos_valid = True
         return self._world_origin
 
@@ -155,8 +144,10 @@ class WorldObject(CountedObject):
         if not self._dir_valid:
             world_dir = np.matmul(self._world_coordinate_transform, self._obj_direction)
             norm = linalg.norm(world_dir)
-            if norm < 1E-7:
-                raise ValueError(f"Measured Norm of World primitives.Vector below tolerance: {norm}")
+            if norm < 1e-7:
+                raise ValueError(
+                    f"Measured Norm of World primitives.Vector below tolerance: {norm}"
+                )
             else:
                 self._world_direction = world_dir / norm
 
@@ -180,7 +171,9 @@ class WorldObject(CountedObject):
     def _get_object_transform(self):
         # if the object transform matrix is out of date, update it
         if not self._obj_transform_valid:
-            self._object_coordinate_transform = np.linalg.inv(self._world_coordinate_transform)
+            self._object_coordinate_transform = np.linalg.inv(
+                self._world_coordinate_transform
+            )
             self._obj_transform_valid = True
         return self._object_coordinate_transform
 
@@ -293,7 +286,9 @@ class ObjectGroup(WorldObject, collections.UserList):
         super().__init__(*args, **kwargs)
 
     def _append_world_transform(self, new_transform):
-        super()._append_world_transform(new_transform)  # update the Groups world transform matrix
+        super()._append_world_transform(
+            new_transform
+        )  # update the Groups world transform matrix
 
         # iterate over all surfaces in the group and update their transform matrices
         for surface in self.data:
@@ -333,7 +328,7 @@ class Intersectable(WorldObject, abc.ABC):
         Returns a tuple of surface ids and surface that make up the intersectable object, used for calculating
         intersections.
         """
-        return (id(self), self),
+        return ((self.get_id(), self),)
 
     @property
     def bounding_volume(self):
@@ -345,7 +340,9 @@ class TracerSurface(Intersectable, abc.ABC):
 
     def __init__(self, surface_args, material=BLACK, *args, **kwargs):
         super().__init__(*args, **kwargs)  # call the next constructor in the MRO
-        self._surface_primitive = type(self).surface(*surface_args)  # create a surface primitive from the provided args
+        self._surface_primitive = type(self).surface(
+            *surface_args
+        )  # create a surface primitive from the provided args
         self.material = material
 
         # make a bounding volume to enclose the shape
@@ -353,7 +350,12 @@ class TracerSurface(Intersectable, abc.ABC):
         self.var_watchlist.append(self._boundary_box_update_fn)
 
     def _boundary_box_update_fn(self):
-        self._aobb = bounding_box(np.matmul(self._world_coordinate_transform, self._surface_primitive.bounding_points))
+        self._aobb = bounding_box(
+            np.matmul(
+                self._world_coordinate_transform,
+                self._surface_primitive.bounding_points,
+            )
+        )
 
     def intersect(self, rays):
         """
@@ -362,8 +364,9 @@ class TracerSurface(Intersectable, abc.ABC):
         :param rays:
         :return:
         """
-        local_ray_set = np.matmul(self._get_object_transform(),
-                                  np.atleast_3d(rays))  # translate the rays into object space
+        local_ray_set = np.matmul(
+            self._get_object_transform(), np.atleast_3d(rays)
+        )  # translate the rays into object space
 
         # get the hits matrix, which is mxn where n is the number of rays propagated
         hits = self._surface_primitive.intersect(local_ray_set)
@@ -375,8 +378,9 @@ class TracerSurface(Intersectable, abc.ABC):
         # hits = np.min(np.where(hits >= 0, hits, np.inf), axis=0)  # reduce the hits to a 1xn array of minimum hits
         # hits = np.where(np.isfinite(hits), hits, -1)  # mask the hits so that anywhere it's np.inf it's cast as -1
 
-        return np.sort(hits, axis=0), np.full(hits.shape,
-                                              id(self))  # return the hits matrix and the intersecting surface
+        return np.sort(hits, axis=0), np.full(
+            hits.shape, self.get_id()
+        )  # return the hits matrix and the intersecting surface
 
     def shade(self, rays: np.ndarray, distances: np.ndarray, **kwargs) -> np.ndarray:
         """
@@ -386,8 +390,12 @@ class TracerSurface(Intersectable, abc.ABC):
 
         # get the normals for the surface at the coordinate
         coordinates = rays[0] + distances * rays[1]
-        normals = self.get_world_normals(coordinates)  # get the world normals of the surface
-        pixel_values = self.material.shade(np.stack((coordinates, rays[1]), axis=0), normals, **kwargs)
+        normals = self.get_world_normals(
+            coordinates
+        )  # get the world normals of the surface
+        pixel_values = self.material.shade(
+            np.stack((coordinates, rays[1]), axis=0), normals, **kwargs
+        )
         return pixel_values
 
     def get_world_normals(self, positions):
@@ -401,9 +409,13 @@ class TracerSurface(Intersectable, abc.ABC):
         local_ray_set = self.to_object_coordinates(positions)
         local_normals = self._surface_primitive.normal(local_ray_set)
         world_normals = np.matmul(self._get_object_transform().T, local_normals)
-        world_normals[-1] = 0  # wipe out any w fields caused by the transpose of the transform
+        world_normals[
+            -1
+        ] = 0  # wipe out any w fields caused by the transpose of the transform
         world_normals /= np.linalg.norm(world_normals, axis=0)
-        return world_normals * self._normal_scale  # return the normals, flipped if the object has them inverted
+        return (
+            world_normals * self._normal_scale
+        )  # return the normals, flipped if the object has them inverted
 
     @property
     def primitive(self) -> primitives.SurfacePrimitive:
@@ -420,33 +432,48 @@ class Sphere(TracerSurface):
 class Cylinder(TracerSurface):
     surface = primitives.Cylinder
 
-    def __init__(self, radius, min_height=-1, max_height=-1, material=BLACK, *args, **kwargs):
-        super().__init__(surface_args=(radius,min_height, max_height), material=material, *args, **kwargs)
+    def __init__(
+        self, radius, min_height=-1, max_height=-1, material=BLACK, *args, **kwargs
+    ):
+        super().__init__(
+            surface_args=(radius, min_height, max_height),
+            material=material,
+            *args,
+            **kwargs,
+        )
 
 
 class Paraboloid(TracerSurface):
     surface = primitives.Paraboloid
 
     def __init__(self, focus=1, height=1, material=BLACK, *args, **kwargs):
-        super().__init__(surface_args=(focus,height), material=material, *args, **kwargs)
+        super().__init__(
+            surface_args=(focus, height), material=material, *args, **kwargs
+        )
 
 
 class XYPlane(TracerSurface):
     surface = primitives.Plane
 
     def __init__(self, width=2, length=2, material=BLACK, *args, **kwargs):
-        super().__init__(surface_args=(width, length), material=material, *args, **kwargs)
+        super().__init__(
+            surface_args=(width, length), material=material, *args, **kwargs
+        )
 
 
 class Cuboid(TracerSurface):
     surface = primitives.Cube
 
-    def __init__(self, l_corner=(-1, -1, -1), r_corner=(1, 1, 1), material=BLACK, *args, **kwargs):
-        super().__init__(surface_args=(l_corner, r_corner), material=material, *args, **kwargs)
+    def __init__(
+        self, l_corner=(-1, -1, -1), r_corner=(1, 1, 1), material=BLACK, *args, **kwargs
+    ):
+        super().__init__(
+            surface_args=(l_corner, r_corner), material=material, *args, **kwargs
+        )
 
     @classmethod
     def from_sides(cls, x=1, y=1, z=1):
-        corners = (np.tile((-0.5, 0.5), (3, 1)).T * np.array((x, y, z)))
+        corners = np.tile((-0.5, 0.5), (3, 1)).T * np.array((x, y, z))
         return cls(*corners)
 
     @classmethod
@@ -458,9 +485,15 @@ class Cuboid(TracerSurface):
 class Cylinder(TracerSurface):
     surface = primitives.Cylinder
 
-    def __init__(self, radius=1, min_height=-1, max_height=1, material=BLACK, *args, **kwargs):
-        super().__init__(surface_args=(radius, min_height, max_height), material=material, *args, **kwargs)
-
+    def __init__(
+        self, radius=1, min_height=-1, max_height=1, material=BLACK, *args, **kwargs
+    ):
+        super().__init__(
+            surface_args=(radius, min_height, max_height),
+            material=material,
+            *args,
+            **kwargs,
+        )
 
 
 class OrthographicCamera(WorldObject):
@@ -468,7 +501,9 @@ class OrthographicCamera(WorldObject):
     A camera oriented along the z-axis pointing along the x-axis
     """
 
-    def __init__(self, h_pixel_count: int, h_width: float, aspect_ratio: float, *args, **kwargs) -> None:
+    def __init__(
+        self, h_pixel_count: int, h_width: float, aspect_ratio: float, *args, **kwargs
+    ) -> None:
         super().__init__(*args, **kwargs)  # call the next constructor in the MRO
         self._h_pixels = h_pixel_count
         self._h_width = h_width
@@ -483,7 +518,9 @@ class OrthographicCamera(WorldObject):
 
     def generate_rays(self) -> np.ndarray:
         rays = self._local_ray_generation()
-        rays = np.matmul(self._world_coordinate_transform, rays)  # transform rays to world space
+        rays = np.matmul(
+            self._world_coordinate_transform, rays
+        )  # transform rays to world space
         rays[1] /= np.linalg.norm(rays[1], axis=0)  # normalize the direction vector
         return rays
 
